@@ -36,3 +36,22 @@ contexts(x_ctx, frequency = "total")
 ## -----------------------------------------------------------------------------
 contexts(x_ctx, frequency = "detailed")
 
+## -----------------------------------------------------------------------------
+node_aaa <- find_sequence(y_ctx_min_1, c("a", "a", "a"))
+node_aaa
+
+## -----------------------------------------------------------------------------
+is_context(node_aaa)
+
+## -----------------------------------------------------------------------------
+children(node_aaa)
+
+## -----------------------------------------------------------------------------
+counts(node_aaa, frequency = "total")
+
+## -----------------------------------------------------------------------------
+counts(node_aaa, frequency = "detailed")
+
+## -----------------------------------------------------------------------------
+positions(node_aaa)
+
